@@ -69,6 +69,20 @@
  * The use count can be changed using g_application_hold() and
  * g_application_release(). If it drops to zero, the application exits.
  *
+ * GApplication also implements the #GActionGroup interface and lets you
+ * easily export actions by adding them with g_application_set_action_group().
+ * When invoking an action by calling g_action_group_activate_action() on
+ * the application, it is always invoked in the primary instance.
+ *
+ * There is a number of different entry points into a #GApplication:
+ * <itemizedlist>
+ * <listitem>via 'Activate' (i.e. just starting the application)</listitem>
+ * <listitem>via 'Open' (i.e. opening some files)</listitem>
+ * <listitem>via activating an action</listitem>
+ * </itemizedlist>
+ * The #GApplication::startup signal lets you handle the application
+ * initialization for all of these in a single place.
+ *
  * <example id="gapplication-example-open"><title>Opening files with a GApplication</title>
  * <programlisting>
  * <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" parse="text" href="../../../../gio/tests/gapplication-example-open.c">
@@ -76,6 +90,14 @@
  * </xi:include>
  * </programlisting>
  * </example>
+ *
+ * <example id="gapplication-example-actions"><title>A GApplication with actions</title>
+ * <programlisting>
+ * <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" parse="text" href="../../../../gio/tests/gapplication-example-actions.c">
+ *   <xi:fallback>FIXME: MISSING XINCLUDE CONTENT</xi:fallback>
+ * </xi:include>
+ * </programlisting>
+ * </example>
  */
 
 struct _GApplicationPrivate
@@ -93,6 +115,7 @@ struct _GApplicationPrivate
   guint              is_registered : 1;
   guint              is_remote : 1;
 
+  GHashTable        *remote_actions;  /* string -> RemoteActionInfo */
   GApplicationImpl  *impl;
 };
 
@@ -226,7 +249,7 @@ g_application_real_local_command_line (GApplication   *application,
 
       if (application->priv->flags & G_APPLICATION_IS_SERVICE)
         {
-          if (n_args > 1)
+          if ((*exit_status = n_args > 1))
             {
               g_printerr ("GApplication service mode takes no arguments.\n");
               application->priv->flags &= ~G_APPLICATION_IS_SERVICE;
@@ -334,7 +357,7 @@ g_application_set_property (GObject      *object,
 /**
  * g_application_set_action_group:
  * @application: a #GApplication
- * @action_group: a #GActionGroup, or %NULL
+ * @action_group: (allow-none): a #GActionGroup, or %NULL
  *
  * Sets or unsets the group of actions associated with the application.
  *
@@ -350,7 +373,7 @@ g_application_set_action_group (GApplication *application,
                                 GActionGroup *action_group)
 {
   g_return_if_fail (G_IS_APPLICATION (application));
-  g_return_if_fail (application->priv->is_registered);
+  g_return_if_fail (!application->priv->is_registered);
 
   if (application->priv->actions != NULL)
     g_object_unref (application->priv->actions);
@@ -522,7 +545,7 @@ g_application_class_init (GApplicationClass *class)
   /**
    * GApplication::open:
    * @application: the application
-   * @files: an array of #GFile objects
+   * @files: (array length=n_files) (element-type GFile): an array of #GFiles
    * @n_files: the length of @files
    * @hint: a hint provided by the calling instance
    *
@@ -570,6 +593,18 @@ get_platform_data (GApplication *application)
     g_free (cwd);
   }
 
+  if (application->priv->flags & G_APPLICATION_SEND_ENVIRONMENT)
+    {
+      GVariant *array;
+      gchar **envp;
+     
+      envp = g_get_environ ();
+      array = g_variant_new_bytestring_array ((const gchar **) envp, -1);
+      g_strfreev (envp);
+
+      g_variant_builder_add (builder, "{sv}", "environ", array);
+    }
+
   G_APPLICATION_GET_CLASS (application)->
     add_platform_data (application, builder);
 
@@ -894,22 +929,21 @@ g_application_register (GApplication  *application,
 
   if (!application->priv->is_registered)
     {
-      gboolean is_remote;
-
       application->priv->impl =
         g_application_impl_register (application, application->priv->id,
                                      application->priv->flags,
-                                     &is_remote, cancellable, error);
+                                     &application->priv->remote_actions,
+                                     cancellable, error);
 
       if (application->priv->impl == NULL)
         return FALSE;
 
-      application->priv->is_remote = is_remote;
+      application->priv->is_remote = application->priv->remote_actions != NULL;
       application->priv->is_registered = TRUE;
 
       g_object_notify (G_OBJECT (application), "is-registered");
 
-      if (!is_remote)
+      if (!application->priv->is_remote)
         g_signal_emit (application, g_application_signals[SIGNAL_STARTUP], 0);
     }
 
@@ -1013,7 +1047,7 @@ g_application_activate (GApplication *application)
 /**
  * g_application_open:
  * @application: a #GApplication
- * @files: an array of #GFiles to open
+ * @files: (array length=n_files): an array of #GFiles to open
  * @n_files: the length of the @files array
  * @hint: a hint (or ""), but never %NULL
  *
@@ -1060,7 +1094,7 @@ g_application_open (GApplication  *application,
  * g_application_run:
  * @application: a #GApplication
  * @argc: the argc from main()
- * @argv: the argv from main()
+ * @argv: (array length=argc): the argv from main()
  * @returns: the exit status
  *
  * Runs the application.
@@ -1198,6 +1232,10 @@ g_application_has_action (GActionGroup *action_group,
 
   g_return_val_if_fail (application->priv->is_registered, FALSE);
 
+  if (application->priv->remote_actions != NULL)
+    return g_hash_table_lookup (application->priv->remote_actions,
+                                action_name) != NULL;
+
   return application->priv->actions &&
          g_action_group_has_action (application->priv->actions, action_name);
 }
@@ -1209,7 +1247,26 @@ g_application_list_actions (GActionGroup *action_group)
 
   g_return_val_if_fail (application->priv->is_registered, NULL);
 
-  if (application->priv->actions != NULL)
+  if (application->priv->remote_actions != NULL)
+    {
+      GHashTableIter iter;
+      gint n, i = 0;
+      gchar **keys;
+      gpointer key;
+
+      n = g_hash_table_size (application->priv->remote_actions);
+      keys = g_new (gchar *, n + 1);
+
+      g_hash_table_iter_init (&iter, application->priv->remote_actions);
+      while (g_hash_table_iter_next (&iter, &key, NULL))
+        keys[i++] = g_strdup (key);
+      g_assert_cmpint (i, ==, n);
+      keys[n] = NULL;
+
+      return keys;
+    }
+
+  else if (application->priv->actions != NULL)
     return g_action_group_list_actions (application->priv->actions);
 
   else
@@ -1226,6 +1283,16 @@ g_application_get_action_enabled (GActionGroup *action_group,
   g_return_val_if_fail (application->priv->actions != NULL, FALSE);
   g_return_val_if_fail (application->priv->is_registered, FALSE);
 
+  if (application->priv->remote_actions)
+    {
+      RemoteActionInfo *info;
+
+      info = g_hash_table_lookup (application->priv->remote_actions,
+                                  action_name);
+
+      return info && info->enabled;
+    }
+
   return g_action_group_get_action_enabled (application->priv->actions,
                                             action_name);
 }
@@ -1239,6 +1306,19 @@ g_application_get_action_parameter_type (GActionGroup *action_group,
   g_return_val_if_fail (application->priv->actions != NULL, NULL);
   g_return_val_if_fail (application->priv->is_registered, NULL);
 
+  if (application->priv->remote_actions)
+    {
+      RemoteActionInfo *info;
+
+      info = g_hash_table_lookup (application->priv->remote_actions,
+                                  action_name);
+
+      if (info)
+        return info->parameter_type;
+      else
+        return NULL;
+    }
+
   return g_action_group_get_action_parameter_type (application->priv->actions,
                                                    action_name);
 }
@@ -1252,23 +1332,23 @@ g_application_get_action_state_type (GActionGroup *action_group,
   g_return_val_if_fail (application->priv->actions != NULL, NULL);
   g_return_val_if_fail (application->priv->is_registered, NULL);
 
+  if (application->priv->remote_actions)
+    {
+      RemoteActionInfo *info;
+
+      info = g_hash_table_lookup (application->priv->remote_actions,
+                                  action_name);
+
+      if (info && info->state)
+        return g_variant_get_type (info->state);
+      else
+        return NULL;
+    }
+
   return g_action_group_get_action_state_type (application->priv->actions,
                                                action_name);
 }
 
-static GVariant *
-g_application_get_action_state_hint (GActionGroup *action_group,
-                                     const gchar  *action_name)
-{
-  GApplication *application = G_APPLICATION (action_group);
-
-  g_return_val_if_fail (application->priv->actions != NULL, NULL);
-  g_return_val_if_fail (application->priv->is_registered, NULL);
-
-  return g_action_group_get_action_state_hint (application->priv->actions,
-                                               action_name);
-}
-
 static GVariant *
 g_application_get_action_state (GActionGroup *action_group,
                                 const gchar  *action_name)
@@ -1278,6 +1358,19 @@ g_application_get_action_state (GActionGroup *action_group,
   g_return_val_if_fail (application->priv->actions != NULL, NULL);
   g_return_val_if_fail (application->priv->is_registered, NULL);
 
+  if (application->priv->remote_actions)
+    {
+      RemoteActionInfo *info;
+
+      info = g_hash_table_lookup (application->priv->remote_actions,
+                                  action_name);
+
+      if (info && info->state)
+        return g_variant_ref (info->state);
+      else
+        return NULL;
+    }
+
   return g_action_group_get_action_state (application->priv->actions,
                                           action_name);
 }
@@ -1289,11 +1382,16 @@ g_application_change_action_state (GActionGroup *action_group,
 {
   GApplication *application = G_APPLICATION (action_group);
 
-  g_return_if_fail (application->priv->actions != NULL);
   g_return_if_fail (application->priv->is_registered);
 
-  g_action_group_change_action_state (application->priv->actions,
-                                      action_name, value);
+  if (application->priv->is_remote)
+    g_application_impl_change_action_state (application->priv->impl,
+                                            action_name, value,
+                                            get_platform_data (application));
+
+  else
+    g_action_group_change_action_state (application->priv->actions,
+                                        action_name, value);
 }
 
 static void
@@ -1303,11 +1401,16 @@ g_application_activate_action (GActionGroup *action_group,
 {
   GApplication *application = G_APPLICATION (action_group);
 
-  g_return_if_fail (application->priv->actions != NULL);
   g_return_if_fail (application->priv->is_registered);
 
-  g_action_group_activate_action (application->priv->actions,
-                                  action_name, parameter);
+  if (application->priv->is_remote)
+    g_application_impl_activate_action (application->priv->impl,
+                                        action_name, parameter,
+                                        get_platform_data (application));
+
+  else
+    g_action_group_activate_action (application->priv->actions,
+                                    action_name, parameter);
 }
 
 static void
@@ -1319,7 +1422,6 @@ g_application_action_group_iface_init (GActionGroupInterface *iface)
   iface->get_action_enabled = g_application_get_action_enabled;
   iface->get_action_parameter_type = g_application_get_action_parameter_type;
   iface->get_action_state_type = g_application_get_action_state_type;
-  iface->get_action_state_hint = g_application_get_action_state_hint;
   iface->get_action_state = g_application_get_action_state;
   iface->change_action_state = g_application_change_action_state;
   iface->activate_action = g_application_activate_action;
