@@ -28,8 +28,12 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef _WIN32
+#include <io.h>
+#endif
 
 #include "giotypes.h"
+#include "gioerror.h"
 #include "gdbusaddress.h"
 #include "gdbusutils.h"
 #include "gdbusconnection.h"
@@ -40,13 +44,18 @@
 #include "gio-marshal.h"
 #include "ginitable.h"
 #include "gsocketservice.h"
+#include "gthreadedsocketservice.h"
+#include "gresolver.h"
+#include "ginetaddress.h"
+#include "ginetsocketaddress.h"
+#include "ginputstream.h"
+#include "giostream.h"
 
 #ifdef G_OS_UNIX
 #include "gunixsocketaddress.h"
 #endif
 
 #include "glibintl.h"
-#include "gioalias.h"
 
 /**
  * SECTION:gdbusserver
@@ -59,8 +68,19 @@
  * <example id="gdbus-peer-to-peer"><title>D-Bus peer-to-peer example</title><programlisting><xi:include xmlns:xi="http://www.w3.org/2001/XInclude" parse="text" href="../../../../gio/tests/gdbus-example-peer.c"><xi:fallback>FIXME: MISSING XINCLUDE CONTENT</xi:fallback></xi:include></programlisting></example>
  */
 
-struct _GDBusServerPrivate
+/**
+ * GDBusServer:
+ *
+ * The #GDBusServer structure contains only private data and
+ * should only be accessed using the provided API.
+ *
+ * Since: 2.26
+ */
+struct _GDBusServer
 {
+  /*< private >*/
+  GObject parent_instance;
+
   GDBusServerFlags flags;
   gchar *address;
   gchar *guid;
@@ -72,6 +92,7 @@ struct _GDBusServerPrivate
 
   GSocketListener *listener;
   gboolean is_using_listener;
+  gulong run_signal_handler_id;
 
   /* The result of g_main_context_get_thread_default() when the object
    * was created (the GObject _init() function) - this is used for delivery
@@ -84,6 +105,27 @@ struct _GDBusServerPrivate
   GDBusAuthObserver *authentication_observer;
 };
 
+typedef struct _GDBusServerClass GDBusServerClass;
+
+/**
+ * GDBusServerClass:
+ * @new_connection: Signal class handler for the #GDBusServer::new-connection signal.
+ *
+ * Class structure for #GDBusServer.
+ *
+ * Since: 2.26
+ */
+struct _GDBusServerClass
+{
+  /*< private >*/
+  GObjectClass parent_class;
+
+  /*< public >*/
+  /* Signals */
+  gboolean (*new_connection) (GDBusServer      *server,
+                              GDBusConnection  *connection);
+};
+
 enum
 {
   PROP_0,
@@ -114,27 +156,30 @@ g_dbus_server_finalize (GObject *object)
 {
   GDBusServer *server = G_DBUS_SERVER (object);
 
-  if (server->priv->authentication_observer != NULL)
-    g_object_unref (server->priv->authentication_observer);
+  if (server->authentication_observer != NULL)
+    g_object_unref (server->authentication_observer);
 
-  if (server->priv->listener != NULL)
-    g_object_unref (server->priv->listener);
+  if (server->run_signal_handler_id > 0)
+    g_signal_handler_disconnect (server->listener, server->run_signal_handler_id);
 
-  g_free (server->priv->address);
-  g_free (server->priv->guid);
-  g_free (server->priv->client_address);
-  if (server->priv->nonce != NULL)
+  if (server->listener != NULL)
+    g_object_unref (server->listener);
+
+  g_free (server->address);
+  g_free (server->guid);
+  g_free (server->client_address);
+  if (server->nonce != NULL)
     {
-      memset (server->priv->nonce, '\0', 16);
-      g_free (server->priv->nonce);
+      memset (server->nonce, '\0', 16);
+      g_free (server->nonce);
     }
   /* we could unlink the nonce file but I don't
    * think it's really worth the effort/risk
    */
-  g_free (server->priv->nonce_file);
+  g_free (server->nonce_file);
 
-  if (server->priv->main_context_at_construction != NULL)
-    g_main_context_unref (server->priv->main_context_at_construction);
+  if (server->main_context_at_construction != NULL)
+    g_main_context_unref (server->main_context_at_construction);
 
   G_OBJECT_CLASS (g_dbus_server_parent_class)->finalize (object);
 }
@@ -150,27 +195,27 @@ g_dbus_server_get_property (GObject    *object,
   switch (prop_id)
     {
     case PROP_FLAGS:
-      g_value_set_flags (value, server->priv->flags);
+      g_value_set_flags (value, server->flags);
       break;
 
     case PROP_GUID:
-      g_value_set_string (value, server->priv->guid);
+      g_value_set_string (value, server->guid);
       break;
 
     case PROP_ADDRESS:
-      g_value_set_string (value, server->priv->address);
+      g_value_set_string (value, server->address);
       break;
 
     case PROP_CLIENT_ADDRESS:
-      g_value_set_string (value, server->priv->client_address);
+      g_value_set_string (value, server->client_address);
       break;
 
     case PROP_ACTIVE:
-      g_value_set_boolean (value, server->priv->active);
+      g_value_set_boolean (value, server->active);
       break;
 
     case PROP_AUTHENTICATION_OBSERVER:
-      g_value_set_object (value, server->priv->authentication_observer);
+      g_value_set_object (value, server->authentication_observer);
       break;
 
     default:
@@ -190,19 +235,19 @@ g_dbus_server_set_property (GObject      *object,
   switch (prop_id)
     {
     case PROP_FLAGS:
-      server->priv->flags = g_value_get_flags (value);
+      server->flags = g_value_get_flags (value);
       break;
 
     case PROP_GUID:
-      server->priv->guid = g_value_dup_string (value);
+      server->guid = g_value_dup_string (value);
       break;
 
     case PROP_ADDRESS:
-      server->priv->address = g_value_dup_string (value);
+      server->address = g_value_dup_string (value);
       break;
 
     case PROP_AUTHENTICATION_OBSERVER:
-      server->priv->authentication_observer = g_value_dup_object (value);
+      server->authentication_observer = g_value_dup_object (value);
       break;
 
     default:
@@ -308,14 +353,14 @@ g_dbus_server_class_init (GDBusServerClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_ACTIVE,
-                                   g_param_spec_string ("active",
-                                                        P_("Active"),
-                                                        P_("Whether the server is currently active"),
-                                                        NULL,
-                                                        G_PARAM_READABLE |
-                                                        G_PARAM_STATIC_NAME |
-                                                        G_PARAM_STATIC_BLURB |
-                                                        G_PARAM_STATIC_NICK));
+                                   g_param_spec_boolean ("active",
+                                                         P_("Active"),
+                                                         P_("Whether the server is currently active"),
+                                                         FALSE,
+                                                         G_PARAM_READABLE |
+                                                         G_PARAM_STATIC_NAME |
+                                                         G_PARAM_STATIC_BLURB |
+                                                         G_PARAM_STATIC_NICK));
 
   /**
    * GDBusServer:authentication-observer:
@@ -346,10 +391,12 @@ g_dbus_server_class_init (GDBusServerClass *klass)
    * g_dbus_connection_get_peer_credentials() to figure out what
    * identity (if any), was authenticated.
    *
-   * If you want to accept the connection, simply ref the @connection
-   * object. Then call g_dbus_connection_close() and unref it when you
-   * are done with it. A typical thing to do when accepting a
-   * connection is to listen to the #GDBusConnection::closed signal.
+   * If you want to accept the connection, take a reference to the
+   * @connection object and return %TRUE. When you are done with the
+   * connection call g_dbus_connection_close() and give up your
+   * reference. Note that the other peer may disconnect at any time -
+   * a typical thing to do when accepting a connection is to listen to
+   * the #GDBusConnection::closed signal.
    *
    * If #GDBusServer:flags contains %G_DBUS_SERVER_FLAGS_RUN_IN_THREAD
    * then the signal is emitted in a new thread dedicated to the
@@ -357,31 +404,34 @@ g_dbus_server_class_init (GDBusServerClass *klass)
    * linkend="g-main-context-push-thread-default">thread-default main
    * loop</link> of the thread that @server was constructed in.
    *
+   * You are guaranteed that signal handlers for this signal runs
+   * before incoming messages on @connection are processed. This means
+   * that it's suitable to call g_dbus_connection_register_object() or
+   * similar from the signal handler.
+   *
+   * Returns: %TRUE to claim @connection, %FALSE to let other handlers
+   * run.
+   *
    * Since: 2.26
    */
   _signals[NEW_CONNECTION_SIGNAL] = g_signal_new ("new-connection",
                                                   G_TYPE_DBUS_SERVER,
                                                   G_SIGNAL_RUN_LAST,
                                                   G_STRUCT_OFFSET (GDBusServerClass, new_connection),
-                                                  NULL,
-                                                  NULL,
-                                                  g_cclosure_marshal_VOID__OBJECT,
-                                                  G_TYPE_NONE,
+                                                  g_signal_accumulator_true_handled,
+                                                  NULL, /* accu_data */
+                                                  _gio_marshal_BOOLEAN__OBJECT,
+                                                  G_TYPE_BOOLEAN,
                                                   1,
                                                   G_TYPE_DBUS_CONNECTION);
-
-
-  g_type_class_add_private (klass, sizeof (GDBusServerPrivate));
 }
 
 static void
 g_dbus_server_init (GDBusServer *server)
 {
-  server->priv = G_TYPE_INSTANCE_GET_PRIVATE (server, G_TYPE_DBUS_SERVER, GDBusServerPrivate);
-
-  server->priv->main_context_at_construction = g_main_context_get_thread_default ();
-  if (server->priv->main_context_at_construction != NULL)
-    g_main_context_ref (server->priv->main_context_at_construction);
+  server->main_context_at_construction = g_main_context_get_thread_default ();
+  if (server->main_context_at_construction != NULL)
+    g_main_context_ref (server->main_context_at_construction);
 }
 
 static gboolean
@@ -447,11 +497,11 @@ g_dbus_server_new_sync (const gchar        *address,
   if (server != NULL)
     {
       /* Right now we don't have any transport not using the listener... */
-      g_assert (server->priv->is_using_listener);
-      g_signal_connect (G_SOCKET_SERVICE (server->priv->listener),
-                        "run",
-                        G_CALLBACK (on_run),
-                        server);
+      g_assert (server->is_using_listener);
+      server->run_signal_handler_id = g_signal_connect (G_SOCKET_SERVICE (server->listener),
+                                                        "run",
+                                                        G_CALLBACK (on_run),
+                                                        server);
     }
 
   return server;
@@ -473,7 +523,7 @@ const gchar *
 g_dbus_server_get_client_address (GDBusServer *server)
 {
   g_return_val_if_fail (G_IS_DBUS_SERVER (server), NULL);
-  return server->priv->client_address;
+  return server->client_address;
 }
 
 /**
@@ -490,7 +540,7 @@ const gchar *
 g_dbus_server_get_guid (GDBusServer *server)
 {
   g_return_val_if_fail (G_IS_DBUS_SERVER (server), NULL);
-  return server->priv->guid;
+  return server->guid;
 }
 
 /**
@@ -507,7 +557,7 @@ GDBusServerFlags
 g_dbus_server_get_flags (GDBusServer *server)
 {
   g_return_val_if_fail (G_IS_DBUS_SERVER (server), G_DBUS_SERVER_FLAGS_NONE);
-  return server->priv->flags;
+  return server->flags;
 }
 
 /**
@@ -524,7 +574,7 @@ gboolean
 g_dbus_server_is_active (GDBusServer *server)
 {
   g_return_val_if_fail (G_IS_DBUS_SERVER (server), G_DBUS_SERVER_FLAGS_NONE);
-  return server->priv->active;
+  return server->active;
 }
 
 /**
@@ -539,12 +589,12 @@ void
 g_dbus_server_start (GDBusServer *server)
 {
   g_return_if_fail (G_IS_DBUS_SERVER (server));
-  if (server->priv->active)
+  if (server->active)
     return;
   /* Right now we don't have any transport not using the listener... */
-  g_assert (server->priv->is_using_listener);
-  g_socket_service_start (G_SOCKET_SERVICE (server->priv->listener));
-  server->priv->active = TRUE;
+  g_assert (server->is_using_listener);
+  g_socket_service_start (G_SOCKET_SERVICE (server->listener));
+  server->active = TRUE;
   g_object_notify (G_OBJECT (server), "active");
 }
 
@@ -560,12 +610,15 @@ void
 g_dbus_server_stop (GDBusServer *server)
 {
   g_return_if_fail (G_IS_DBUS_SERVER (server));
-  if (!server->priv->active)
+  if (!server->active)
     return;
   /* Right now we don't have any transport not using the listener... */
-  g_assert (server->priv->is_using_listener);
-  g_socket_service_stop (G_SOCKET_SERVICE (server->priv->listener));
-  server->priv->active = FALSE;
+  g_assert (server->is_using_listener);
+  g_assert (server->run_signal_handler_id > 0);
+  g_signal_handler_disconnect (server->listener, server->run_signal_handler_id);
+  server->run_signal_handler_id = 0;
+  g_socket_service_stop (G_SOCKET_SERVICE (server->listener));
+  server->active = FALSE;
   g_object_notify (G_OBJECT (server), "active");
 }
 
@@ -633,7 +686,7 @@ try_unix (GDBusServer  *server,
       g_string_free (s, TRUE);
 
       local_error = NULL;
-      if (!g_socket_listener_add_address (server->priv->listener,
+      if (!g_socket_listener_add_address (server->listener,
                                           address,
                                           G_SOCKET_TYPE_STREAM,
                                           G_SOCKET_PROTOCOL_DEFAULT,
@@ -671,7 +724,7 @@ try_unix (GDBusServer  *server,
       g_assert_not_reached ();
     }
 
-  if (!g_socket_listener_add_address (server->priv->listener,
+  if (!g_socket_listener_add_address (server->listener,
                                       address,
                                       G_SOCKET_TYPE_STREAM,
                                       G_SOCKET_PROTOCOL_DEFAULT,
@@ -689,18 +742,18 @@ try_unix (GDBusServer  *server,
       /* Fill out client_address if the connection attempt worked */
       if (ret)
         {
-          server->priv->is_using_listener = TRUE;
+          server->is_using_listener = TRUE;
 
           switch (g_unix_socket_address_get_address_type (G_UNIX_SOCKET_ADDRESS (address)))
             {
             case G_UNIX_SOCKET_ADDRESS_ABSTRACT:
-              server->priv->client_address = g_strdup_printf ("unix:abstract=%s",
-                                                              g_unix_socket_address_get_path (G_UNIX_SOCKET_ADDRESS (address)));
+              server->client_address = g_strdup_printf ("unix:abstract=%s",
+                                                        g_unix_socket_address_get_path (G_UNIX_SOCKET_ADDRESS (address)));
               break;
 
             case G_UNIX_SOCKET_ADDRESS_PATH:
-              server->priv->client_address = g_strdup_printf ("unix:path=%s",
-                                                              g_unix_socket_address_get_path (G_UNIX_SOCKET_ADDRESS (address)));
+              server->client_address = g_strdup_printf ("unix:path=%s",
+                                                        g_unix_socket_address_get_path (G_UNIX_SOCKET_ADDRESS (address)));
               break;
 
             default:
@@ -775,7 +828,7 @@ try_tcp (GDBusServer  *server,
       GSocketAddress *effective_address;
 
       socket_address = g_inet_socket_address_new (address, port_num);
-      if (!g_socket_listener_add_address (server->priv->listener,
+      if (!g_socket_listener_add_address (server->listener,
                                           socket_address,
                                           G_SOCKET_TYPE_STREAM,
                                           G_SOCKET_PROTOCOL_TCP,
@@ -801,15 +854,15 @@ try_tcp (GDBusServer  *server,
       gsize bytes_written;
       gsize bytes_remaining;
 
-      server->priv->nonce = g_new0 (guchar, 16);
+      server->nonce = g_new0 (guchar, 16);
       for (n = 0; n < 16; n++)
-        server->priv->nonce[n] = g_random_int_range (0, 256);
+        server->nonce[n] = g_random_int_range (0, 256);
       fd = g_file_open_tmp ("gdbus-nonce-file-XXXXXX",
-                            &server->priv->nonce_file,
+                            &server->nonce_file,
                             error);
       if (fd == -1)
         {
-          g_socket_listener_close (server->priv->listener);
+          g_socket_listener_close (server->listener);
           goto out;
         }
     again:
@@ -818,7 +871,7 @@ try_tcp (GDBusServer  *server,
       while (bytes_remaining > 0)
         {
           gssize ret;
-          ret = write (fd, server->priv->nonce + bytes_written, bytes_remaining);
+          ret = write (fd, server->nonce + bytes_written, bytes_remaining);
           if (ret == -1)
             {
               if (errno == EINTR)
@@ -827,7 +880,7 @@ try_tcp (GDBusServer  *server,
                            G_IO_ERROR,
                            g_io_error_from_errno (errno),
                            _("Error writing nonce file at `%s': %s"),
-                           server->priv->nonce_file,
+                           server->nonce_file,
                            strerror (errno));
               goto out;
             }
@@ -835,16 +888,16 @@ try_tcp (GDBusServer  *server,
           bytes_remaining -= ret;
         }
       close (fd);
-      server->priv->client_address = g_strdup_printf ("nonce-tcp:host=%s,port=%d,noncefile=%s",
-                                                      host,
-                                                      port_num,
-                                                      server->priv->nonce_file);
+      server->client_address = g_strdup_printf ("nonce-tcp:host=%s,port=%d,noncefile=%s",
+                                                host,
+                                                port_num,
+                                                server->nonce_file);
     }
   else
     {
-      server->priv->client_address = g_strdup_printf ("tcp:host=%s,port=%d", host, port_num);
+      server->client_address = g_strdup_printf ("tcp:host=%s,port=%d", host, port_num);
     }
-  server->priv->is_using_listener = TRUE;
+  server->is_using_listener = TRUE;
   ret = TRUE;
 
  out:
@@ -874,11 +927,17 @@ static gboolean
 emit_new_connection_in_idle (gpointer user_data)
 {
   EmitIdleData *data = user_data;
+  gboolean claimed;
 
+  claimed = FALSE;
   g_signal_emit (data->server,
                  _signals[NEW_CONNECTION_SIGNAL],
                  0,
-                 data->connection);
+                 data->connection,
+                 &claimed);
+
+  if (claimed)
+    g_dbus_connection_start_message_processing (data->connection);
   g_object_unref (data->connection);
 
   return FALSE;
@@ -895,7 +954,7 @@ on_run (GSocketService    *service,
   GDBusConnection *connection;
   GDBusConnectionFlags connection_flags;
 
-  if (server->priv->nonce != NULL)
+  if (server->nonce != NULL)
     {
       gchar buf[16];
       gsize bytes_read;
@@ -911,29 +970,32 @@ on_run (GSocketService    *service,
       if (bytes_read != 16)
         goto out;
 
-      if (memcmp (buf, server->priv->nonce, 16) != 0)
+      if (memcmp (buf, server->nonce, 16) != 0)
         goto out;
     }
 
-  connection_flags = G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_SERVER;
-  if (server->priv->flags & G_DBUS_SERVER_FLAGS_AUTHENTICATION_ALLOW_ANONYMOUS)
+  connection_flags =
+    G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_SERVER |
+    G_DBUS_CONNECTION_FLAGS_DELAY_MESSAGE_PROCESSING;
+  if (server->flags & G_DBUS_SERVER_FLAGS_AUTHENTICATION_ALLOW_ANONYMOUS)
     connection_flags |= G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_ALLOW_ANONYMOUS;
 
   connection = g_dbus_connection_new_sync (G_IO_STREAM (socket_connection),
-                                           server->priv->guid,
+                                           server->guid,
                                            connection_flags,
-                                           server->priv->authentication_observer,
+                                           server->authentication_observer,
                                            NULL,  /* GCancellable */
                                            NULL); /* GError */
   if (connection == NULL)
       goto out;
 
-  if (server->priv->flags & G_DBUS_SERVER_FLAGS_RUN_IN_THREAD)
+  if (server->flags & G_DBUS_SERVER_FLAGS_RUN_IN_THREAD)
     {
       g_signal_emit (server,
                      _signals[NEW_CONNECTION_SIGNAL],
                      0,
                      connection);
+      g_dbus_connection_start_message_processing (connection);
       g_object_unref (connection);
     }
   else
@@ -951,7 +1013,7 @@ on_run (GSocketService    *service,
                              emit_new_connection_in_idle,
                              data,
                              (GDestroyNotify) emit_idle_data_free);
-      g_source_attach (idle_source, server->priv->main_context_at_construction);
+      g_source_attach (idle_source, server->main_context_at_construction);
       g_source_unref (idle_source);
     }
 
@@ -971,21 +1033,22 @@ initable_init (GInitable     *initable,
   GError *last_error;
 
   ret = FALSE;
+  addr_array = NULL;
   last_error = NULL;
 
-  if (!g_dbus_is_guid (server->priv->guid))
+  if (!g_dbus_is_guid (server->guid))
     {
       g_set_error (&last_error,
                    G_IO_ERROR,
                    G_IO_ERROR_INVALID_ARGUMENT,
                    _("The string `%s' is not a valid D-Bus GUID"),
-                   server->priv->guid);
+                   server->guid);
       goto out;
     }
 
-  server->priv->listener = G_SOCKET_LISTENER (g_threaded_socket_service_new (-1));
+  server->listener = G_SOCKET_LISTENER (g_threaded_socket_service_new (-1));
 
-  addr_array = g_strsplit (server->priv->address, ";", 0);
+  addr_array = g_strsplit (server->address, ";", 0);
   last_error = NULL;
   for (n = 0; addr_array != NULL && addr_array[n] != NULL; n++)
     {
@@ -1044,6 +1107,9 @@ initable_init (GInitable     *initable,
     goto out;
 
  out:
+
+  g_strfreev (addr_array);
+
   if (ret)
     {
       if (last_error != NULL)
@@ -1065,6 +1131,3 @@ initable_iface_init (GInitableIface *initable_iface)
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
-
-#define __G_DBUS_SERVER_C__
-#include "gioaliasdef.c"
